<?php

namespace HCM\FPA\Calculos\FolhaComplementar;

use HCM\FPA\Calculos\Geral\Calculo;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\TkDI;
use HCM\Util\MsgCalculo;
use HCM\FPA\Calculos\Geral\MemoriaCalculo;
use HCM\FPA\Entity\FpatipoincideEntity;
use HCM\FPA\Entity\FpaProcessoEntity;
use HCM\FPA\Entity\FpaPtregraEntity;
use HCM\FPA\Entity\FpaValorreferenciamEntity;
use HCM\FPA\Entity\FpaincideventoEntity;
use HCM\Geral\Entity\GpesitufuncmEntity;
use HCM\Geral\Entity\GpevinculohEntity;
use HCM\FPA\Entity\FpaParamfolhahEntity;
use HCM\FPA\Entity\FpatipomovimentEntity;
use HCM\FPA\Entity\FpaFormcalculoEntity;
use HCM\FPA\Entity\FpatipoassoceveEntity;
use HCM\FPA\Entity\FpaitecalcfolhaEntity;
use HCM\FPA\Entity\FpatipocalculoEntity;
use Teknisa\Model\Gestaopessoas\Repositories\FpaValorreferenciam;
use HCM\FPA\Calculos\Geral\FGTS;
use HCM\FPA\Calculos\Geral\INSS;
use HCM\FPA\Calculos\Geral\IRRF;

class CalculoFolhaComplementar extends Calculo {

    public function calculaFolhaComplementar($vinculo, $tp_calc_mov, $contador, $itemsOrig, $itemsPrincipal, $_mesCompetenciaAtual){
        
        $this->_inicializaVariaveis($vinculo, $tp_calc_mov, $_mesCompetenciaAtual);
        
        $this->_itens_calculo_principal = $itemsPrincipal;
  
        $this->_controleLiquidoParcial();
        
        $this->_inicializaVariaveisRegra();

        $this->_calculo_regra->geraVariaveis();
        
        $this->_calculo_regra->executaPontoRegra(
            $this->_idDebug, FpaPtregraEntity::PTREGRA_INICIO, $this->_nr_tipo_calculo,$this->_itens_calculo_principal
        );

        $this->_verificaTrabalhoEfetivo($vinculo);
        
        $this->_geraReferenciaSitFunc();
        
        $this->_calculaRemuneracaoBase($vinculo, $this->_itens_calculo_principal, $this->_nr_tipo_calculo);
        
        $this->_demonstraAdiantamento($tp_calc_mov);
        
        $this->_demonstraFerias($tp_calc_mov);
        
        $this->_verificaDifFerias($tp_calc_mov, $vinculo, true);
        
        $this->_calculaLicencaRemun();

        $this->_calculaValeTransporte();

        if((int)$this->_sit_func_resumo[GpesitufuncmEntity::SITUACAO_FUNCIONAL_NORMAL]>0 ||
            (int)$this->_sit_func_resumo[GpesitufuncmEntity::SITUACAO_FUNCIONAL_MATERNIDADE]>0)
        {
            $this->_calculaContribuicaoSindical($vinculo);
        }
        
        $this->_analisaSalMaternidade($vinculo, $tp_calc_mov);
        
        $this->_calculo_regra->executaPontoRegra(
            $this->_idDebug, FpaPtregraEntity::PTREGRA_ANTES_CALC_ESPECIFICO, $this->_nr_tipo_calculo,$this->_itens_calculo_principal
        );    
        
        $this->_executaFolha($vinculo);

        $this->_calculaSalarioFamilia($vinculo);

        $this->_calculaGrupoIncidencias();

        $this->_geraPensao();
        
        $incideventoEntity = new FpaincideventoEntity($this->entityManager, $this->hcmParameters);
        $incideInss = $incideventoEntity->getEventosIncDissidio(
            $this->_mes_competencia, FpatipoincideEntity::INC_INSS_CALC
        );
        
        $incideEventosNaoRecalcula = $incideventoEntity->getEventosIncDissidio(
            $this->_mes_competencia, FpatipoincideEntity::NÃO_RECALCULA_DISSIDIO
        );
        
        $incideEventosRecalculaPerc = $incideventoEntity->getEventosIncDissidio(
            $this->_mes_competencia, FpatipoincideEntity::RECALCULA_POR_DISSIDIO
        );
        
        
        //Recalcula evento pela porcentagem do Salario.
        if($incideEventosRecalculaPerc){
            
            $salarioDissidio = $this->_salario;
            $alteSalario = $this->_alteracaoSalario->retornaAlteSalarioAtualPorVinculoModalidade($this->_mes_competencia, $vinculo->getNrvinculom(), 1);
            $salarioMes = $alteSalario->getVrsalario();
            
            $VrMultiplicador = ((($salarioDissidio - $salarioMes) / $salarioMes) * 100) / 100;
            
            foreach($incideEventosRecalculaPerc as $nrEvento){
                #var_dump($nrEvento);
                $evento = $this->_eventom->retornaEvento($nrEvento);
                $indice = $itemsPrincipal->procuraEvento($evento);
                $itemAntigo = $itemsPrincipal->getItem($indice);
                
                if($itemAntigo){
                    $vrEventoAntigo = $itemAntigo->getVreventofolha();
                    $vrNovo = $vrEventoAntigo + ($itemAntigo->getVreventofolha() * $VrMultiplicador);
                    $this->_itens_calculo_principal->geraEvento($evento, null, $vrNovo, 0, 1, 'VALOR');
                }
            }
        }
        
        //Salva valores antigo negativos.
        $eventosInseridos = [];
        foreach($itemsOrig as $itemOrig){
            $item = clone $itemOrig;
            $evento = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaEvento($item->getNreventom(), $this->_nrorg, 0, $this->_mes_competencia);
            
            if(in_array($evento->getNreventom(), $incideEventosNaoRecalcula)){
                continue;   
            }
            
            if(array_key_exists($evento->getNreventom(), $eventosInseridos) && $item->getVreventofolha() < 0){
                $evento = $this->_eventom->retornaEvento($evento->getNreventom());
                $indice = $itemsPrincipal->procuraEvento($evento);
                $itemOld = $itemsPrincipal->getItem($indice);
                $ref = $itemOld->getVrreffolha() + $item->getVrreffolha();
                $val = $itemOld->getVreventofolha() - $item->getVreventofolha();
                $itemOld->setVrreffolha($ref);
                $itemOld->setVreventofolha(-abs($val));
            }else{
                $item->setNritemcalcfolha(\HCM\Util\NovoCodigo::geraCodigo($this->_nrorg, 'FPA_ITECALCFOLHA', 1, 12, true));
                $item->setIdrefevalorcalc('NORMAL');
                $item->setNrcalculofolha($itemsPrincipal->getNrcalculofolha());
                $item->setDtocorrencia(DateUtil::getDataDeString($item->getDtocorrencia()));
                $item->setVreventofolha(-abs($item->getVreventofolha()));
                $itemsPrincipal->addItem($item);
            }
            $eventosInseridos[$evento->getNreventom()] = true;       
            
        }
        
        $this->_calculaTotais(); 
        
        $this->_calculo_regra->executaPontoRegra(
            $this->_idDebug, FpaPtregraEntity::PTREGRA_FINAL, $this->_nr_tipo_calculo,$this->_itens_calculo_principal
        );
        
        $lista = null;
        
        $msgcalculo = MsgCalculo::getInstance()->getDados();
        
        if ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ERRO) {
            return $lista;
        }
        if ($this->_simulacao === false) {
            
            //Evento de liquido (diferenca da competencia).
            $eventoLiquido = $this->_eventom->retornaEvento(FpatipoassoceveEntity::ASSOC_LIQUIDO);
            $liquido = $this->_itens_calculo_principal->retornaValorPorEvento($eventoLiquido, null);
            
            $items = $this->_itens_calculo_principal->getItems();
            $valProv = 0;
            $valProvNaoIncide = 0;
            $valDesc = 0;
            $valDescNaoIncide = 0;
            
            foreach($items as $item){
                $evento = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaEvento($item->getNreventom(), $this->_nrorg, 0, $this->_mes_competencia);
                
                if(in_array($evento->getNreventom(), $incideEventosNaoRecalcula)){
                    continue;
                }
                
                if(in_array($item->getNreventom(), $incideInss)){
                    if($evento->getIdtipoevento() == 'PROVENTO'){
                        $valProv += $item->getVreventofolha();
                    } 
                    if($evento->getIdtipoevento() == 'DESCONTO' ){
                        $valDesc += $item->getVreventofolha();
                    }
                }else{
                    if($evento->getIdtipoevento() == 'PROVENTO'){
                        $valProvNaoIncide += $item->getVreventofolha();
                    } 
                    if($evento->getIdtipoevento() == 'DESCONTO'){
                        $valDescNaoIncide += $item->getVreventofolha();
                    }
                }    
            }
            
            //Busca folha atual para inserir os evento de diferenca
            $_mesCompetenciaAtual = DateUtil::getDataDeString($_mesCompetenciaAtual);   
            $calc_folhaAtual = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->retornaCalculoFolha(
                $this->_nrorg, FpatipomovimentEntity::TP_MOV_MENSAL, 1, 1, $_mesCompetenciaAtual, $_mesCompetenciaAtual
            )[0];
            $itemCalcCompetenciaAtual = new FpaitecalcfolhaEntity($this->entityManager, $this->hcmParameters, $calc_folhaAtual->getNrcalculofolha());
            $itemCalcCompetenciaAtual->retornaItensPorVinculo($vinculo->getNrorg(), $vinculo->getNrvinculom(), $calc_folhaAtual->getNrcalculofolha());
            
            //Verifica Salario Maternidade
            $afastamento = $this->_verificaAfastamentos(
               $vinculo, $this->_situacao_funcional, $_mesCompetenciaAtual, DateUtil::getUltimoDiaDoMes($_mesCompetenciaAtual)
            );
            if(in_array(GpesitufuncmEntity::SITUACAO_FUNCIONAL_MATERNIDADE, $afastamento)) {
                //Gera evento de diferenca provento na folha atual.
                $evento_difSalarioMaternidade = $this->_eventom->retornaEvento(334);
                $valDifSalario = $itemCalcCompetenciaAtual->retornaValorPorEvento($evento_difSalarioMaternidade, null);
                if($valDifSalario > 0 and $valProv > 0){
                    $indice = $itemCalcCompetenciaAtual->procuraEvento($evento_difSalarioMaternidade);
                    $lanca_difSalario = $itemCalcCompetenciaAtual->getItem($indice);
                    $val_difSalario = $lanca_difSalario->getVreventofolha() + $valProv;
                    if($lanca_difSalario->getIdrefevalorcalc() == 'VALOR'){
                        $lanca_difSalario->setVreventofolha($val_difSalario);
                        $this->entityManager->persist($lanca_difSalario);
                        $this->entityManager->flush($lanca_difSalario);
                    }
                }else if($valProv > 0){
                    if($contador>0){
                        $valProvMaterDiv = $valProv/$contador;
                        $itemCalcCompetenciaAtual->geraEvento($evento_difSalarioMaternidade, null, $valProvMaterDiv, 0, $contador, 'VALOR');
                        $itemCalcCompetenciaAtual->persist();
                    }else{
                        $itemCalcCompetenciaAtual->geraEvento($evento_difSalarioMaternidade, null, $valProv, 0, 1, 'VALOR');
                        $itemCalcCompetenciaAtual->persist();
                    }
                }
            }else{
                //Gera evento de diferenca provento na folha atual.
                $evento_difSalarioProv = $this->_eventom->retornaEvento(FpatipoassoceveEntity::ASSOC_DIF_SALARIO_DISSIDIO);
                $valDifSalario = $itemCalcCompetenciaAtual->retornaValorPorEvento($evento_difSalarioProv, null);
                if($valDifSalario > 0 and $valProv > 0){
                    $indice = $itemCalcCompetenciaAtual->procuraEvento($evento_difSalarioProv);
                    $lanca_difSalario = $itemCalcCompetenciaAtual->getItem($indice);
                    $val_difSalario = $lanca_difSalario->getVreventofolha() + $valProv;
                    if($lanca_difSalario->getIdrefevalorcalc() == 'VALOR'){
                        $lanca_difSalario->setVreventofolha($val_difSalario);
                        $this->entityManager->persist($lanca_difSalario);
                        $this->entityManager->flush($lanca_difSalario);
                    }
                }else if($valProv > 0){
                    if($contador>0){
                        $valProvDiv = $valProv/$contador;
                        $itemCalcCompetenciaAtual->geraEvento($evento_difSalarioProv, null, $valProvDiv, 0, $contador, 'VALOR');
                        $itemCalcCompetenciaAtual->persist();
                    }else{
                        $itemCalcCompetenciaAtual->geraEvento($evento_difSalarioProv, null, $valProv, 0, 1, 'VALOR');
                        $itemCalcCompetenciaAtual->persist();
                    }
                }
            }
            
            //Gera evento de diferenca provento que nao incide para o inss na folha atual.
            $evento_difProvNaoIncide = $this->_eventom->retornaEvento(205510);
            $valDifSalario = $itemCalcCompetenciaAtual->retornaValorPorEvento($evento_difProvNaoIncide, null);
            
            if($valDifSalario > 0 and $valProvNaoIncide > 0){
                $indice = $itemCalcCompetenciaAtual->procuraEvento($evento_difProvNaoIncide);
                $lanca_difSalario = $itemCalcCompetenciaAtual->getItem($indice);
                $val_difSalario = $lanca_difSalario->getVreventofolha() + $valProvNaoIncide;
                if($lanca_difSalario->getIdrefevalorcalc() == 'VALOR'){
                    $lanca_difSalario->setVreventofolha($val_difSalario);
                    $this->entityManager->persist($lanca_difSalario);
                    $this->entityManager->flush($lanca_difSalario);
                }
            }else if($valProvNaoIncide > 0){
                $itemCalcCompetenciaAtual->geraEvento($evento_difProvNaoIncide, null, $valProvNaoIncide, 0, 1, 'VALOR');
                $itemCalcCompetenciaAtual->persist();
            }
            
            //Gera evento de diferenca de desconto na folha atual.
            $evento_difDesconto = $this->_eventom->retornaEvento(205580);
            $valDifSalario = $itemCalcCompetenciaAtual->retornaValorPorEvento($evento_difDesconto, null);
            if($valDifSalario > 0 and $valDesc > 0){
                $indice = $itemCalcCompetenciaAtual->procuraEvento($evento_difDesconto);
                $lanca_difSalario = $itemCalcCompetenciaAtual->getItem($indice);
                $val_difSalario = $lanca_difSalario->getVreventofolha() + $valDesc;
                if($lanca_difSalario->getIdrefevalorcalc() == 'VALOR'){
                    $lanca_difSalario->setVreventofolha($val_difSalario);
                    $this->entityManager->persist($lanca_difSalario);
                    $this->entityManager->flush($lanca_difSalario);
                }
            }else if($valDesc > 0){
                if($contador>0){
                    $valDescDiv = $valDesc/$contador;
                    $itemCalcCompetenciaAtual->geraEvento($evento_difDesconto, null, $valDescDiv, 0, $contador, 'VALOR');
                    $itemCalcCompetenciaAtual->persist();
                }else{
                    $itemCalcCompetenciaAtual->geraEvento($evento_difDesconto, null, $valDesc, 0, 1, 'VALOR');
                    $itemCalcCompetenciaAtual->persist();
                }
            }
            
            //Gera evento de diferenca desconto que nao incide na folha atual.
            $evento_difDescontoNaoIncide = $this->_eventom->retornaEvento(205559);
            $valDifSalario = $itemCalcCompetenciaAtual->retornaValorPorEvento($evento_difDescontoNaoIncide, null);
            if($valDifSalario > 0 and $valDescNaoIncide > 0){
                $indice = $itemCalcCompetenciaAtual->procuraEvento($evento_difDescontoNaoIncide);
                $lanca_difSalario = $itemCalcCompetenciaAtual->getItem($indice);
                $val_difSalario = $lanca_difSalario->getVreventofolha() + $valDescNaoIncide;
                if($lanca_difSalario->getIdrefevalorcalc() == 'VALOR'){
                    $lanca_difSalario->setVreventofolha($val_difSalario);
                    $this->entityManager->persist($lanca_difSalario);
                    $this->entityManager->flush($lanca_difSalario);
                }
            }else if($valDescNaoIncide > 0){
                    $itemCalcCompetenciaAtual->geraEvento($evento_difDescontoNaoIncide, null, $valDescNaoIncide, 0, 1, 'VALOR');
                    $itemCalcCompetenciaAtual->persist();
            }
            
            $tp_calc_mov->persisteItensCalculo();
            
            return $lista;
        } else {
            $lista['SIMULACAO'] = $tp_calc_mov->retornaListaItensCalculo(
                $this->_mes_competencia, $this->_eventom, $vinculo
            );
            //Vinculo
            if(count($lista) > 0){
                $lista['SIMULACAO'][0][] = array(
                    'MESOCORRENCIA' => $vinculo->getNrvinculom(),
                    'TIPOEVENTO' => $vinculo->getNmvinculom(),
                    'NRFILHO' => $vinculo->getNrvinculom(),
                    'NREVENTOM' => 'Matricula: '.$vinculo->getCdmatricula(),
                    'NMEVENTOH' => 'Salario: ' .number_format($this->_salario, 2, ',','.'),
                    'UNIDREFEEVEN' => 'Admissão: '.substr($vinculo->getDtadmissaovinc(), 0, 10),
                    'VRREFFOLHA' => null,
                    'IDEVENDEMON' => null,
                    'VREVENTOFOLHA' => null,
                    'NRORDEIMPRES' => null,
                    'ORDEMTIPOEVENTO' => null,
                    'NRVINCULOM' => null,
                    'NRCALCULOFOLHAVINC' => null
                );    
            }
            
            return $lista;
        }
    }
}