<?php 

namespace HCM\FPA\ObrigacaoLegal\CNAB;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\TkDI;
use HCM\Util\MsgCalculo;
use HCM\Geral\Entity\GpevinculomEntity;
use HCM\Geral\Entity\TipohierarqorgEntity;
use HCM\Geral\Entity\HierarquiaorgEntity;
use HCM\Geral\Entity\GpepessoaEntity;
use HCM\FPA\Calculos\Geral\Estruturacao;
use HCM\FPA\Entity\FpaGrpitemEntity;
use HCM\FPA\Entity\FpacalculofolhaEntity;
use HCM\FPA\Entity\FpaCnabmagnetEntity;
use HCM\FPA\ObrigacaoLegal\CNAB\CNAB;
use HCM\FPA\Registro\RegistroCNAB;
use HCM\FPA\Registro\FPARegistro;

use HCM\FPA\ObrigacaoLegal\CNAB\Santander\registro\Registro5CNABSantander;

class PercorreCNAB{

    private $_formulario;
    private $_retrocesso;
    private $_nrOrg;
    private $_nrOrgPadrao;
    private $_nrEstruturaTrab;
    private $_mesCompetencia;
    private $_dsInicio;
    private $_dsFim;
    private $_dsOrdem;
    private $_nrTpHierarquiaOrg;
    private $_idGeraMeioMagnetico;
    private $_idProcessaVinculo;
    private $_nrEstruturaResp;
    private $_filePath;
    private $_tpInscResp;
    private $_nrEvento;
    private $_nrTipoMovimento;
    private $_cdBanco;
    private $_nrGrupoCnab;
    private $_nrTipoModalidadeCalculo;
    private $_nrOcorrenciaCalculo;
    private $_calculoFolha;
    private $_nrCalculoFolha;
    private $_nrCalculoFolhaLista;
    private $_cnabMagnetico;
    private $_tipoHierarquia;
    private $_hierarquiaOrg;
    private $_vinculom;
    private $_pessoa;
    private $_dtPagamento;
    private $_estruturacao;
    private $_nomeArquivo;
    private $_cnab;
    private $_msgSucessoPersisteMagnetico;
    private $_msgSucessoGeraMagnetico;
    private $_contador;
    private $_idUtilDtPgtoItem;
    private $_idIgnoraOcorrencia;
    private $_idAgrupArquivo;
    private $_tpVinculo;
    private $_cdOperador;

    const PREFIXO_NOME_ARQUIVO_CNAB = 'cnab';
    const EXTENSAO_ARQUIVO = '.txt';

    public function __construct($entityManager, $hcmParameters, $dataSet, $tpvinculo = null){

        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->_nrOrg = $hcmParameters->getNrorgTrabalho();
        $this->_nrOrgPadrao = $hcmParameters->getNrorgPadrao();
        $this->_cdOperador = $hcmParameters->getCdOperador();
        $this->_nrEstruturaTrab = FPA::getEstruturaTrabalho();
        $this->_mesCompetencia = DateUtil::getDataDeString($hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);
        $this->_nrTipoModalidadeCalculo = $hcmParameters->getTipoModalidadeCalculo();
        $this->_nrOcorrenciaCalculo = FPA::getOcorrenciaCalculo();
        $this->_itemGrupo = new FpaGrpitemEntity($entityManager, $hcmParameters);
        $this->_vinculom = new GpevinculomEntity($entityManager, $hcmParameters);
        $this->_tipoHierarquia = new TipohierarqorgEntity($entityManager, $hcmParameters);
        $this->_hierarquiaOrg = new HierarquiaorgEntity($entityManager, $hcmParameters);
        $this->_calculoFolha = new FpacalculofolhaEntity($entityManager, $hcmParameters);
        $this->_cnabMagnetico = new FpaCnabmagnetEntity($entityManager, $hcmParameters);
        $this->_pessoa = new GpepessoaEntity($entityManager, $hcmParameters);
        $this->_estruturacao = new Estruturacao($entityManager, $hcmParameters);
        $this->_inicializaVariaveis($dataSet);
        $this->_tpVinculo = $tpvinculo;

        $tpProcessamento = strtoupper($dataSet['TP_PROCESSAMENTO']);
        switch ($tpProcessamento){
        case FPA::PROC_COMPETENCIA_MENSAL :
             $this->_retrocesso = false;
             break;
        case FPA::PROC_RETROCESSO :
            $this->_retrocesso = true;
            break;
        default:
            throw new \Exception('Tipo de processo '.$tp_processamento.' invalido');
        }
        
        if ($this->_idIgnoraOcorrencia == 'S') {
            $this->_cnab = new CNAB(
                $this->_nrOrg, $this->_nrOrgPadrao, $this->_mesCompetencia, $this->_cnabMagnetico, $dataSet, $this->_nrCalculoFolhaLista, $entityManager, $hcmParameters
            );
        } else {
            $this->_cnab = new CNAB(
                $this->_nrOrg, $this->_nrOrgPadrao, $this->_mesCompetencia, $this->_cnabMagnetico, $dataSet, $this->_nrCalculoFolha, $entityManager, $hcmParameters
            );
        }
    }

    public function _inicializaVariaveis($dataSet) {
        $this->_buscaDadosFormulario($dataSet);

        if ($this->_idIgnoraOcorrencia == 'S') {
            $calculosFolha = $this->_calculoFolha->retornaCalculoFolha(
                $this->_nrOrg, $this->_nrTipoMovimento,$this->_mesCompetencia, $this->_nrTipoModalidadeCalculo, null);
            
            foreach ($calculosFolha as $calculoFolha) {
                $this->_nrCalculoFolhaLista[] = $calculoFolha->getNrcalculofolha();
            }
            
            $this->_nrCalculoFolha = end($calculosFolha)->getNrcalculofolha();
        } else {
            $calculoFolha = $this->_calculoFolha->retornaCalculoFolha(
                $this->_nrOrg, $this->_nrTipoMovimento,$this->_mesCompetencia, $this->_nrTipoModalidadeCalculo, $this->_nrOcorrenciaCalculo);
            
            $this->_nrCalculoFolha = end($calculoFolha)->getNrcalculofolha();
        }

        $this->_msgSucessoPersisteMagnetico = null;
        $this->_msgSucessoGeraMagnetico = null;
    }

    public function percorre(){
        if ($this->_retrocesso === false) {

            if ($this->_idAgrupArquivo == 'ESTRUTGEREN' || $this->_idAgrupArquivo == 'VINCULO') {
                $resultado = $this->_geraCNAB_Agrup();
            } else {
                $resultado = $this->_geraCNAB();
            }
            return $resultado;
        } else {
            $this->_cnab->retroceder();
        }
    }

    private function _geraCNAB(){
        
        $idabort = false;
        if ($this->_idProcessaVinculo === 'S') {
            try{
                $this->_cnabMagnetico->inicializaListaRegistros(
                    $this->_mesCompetencia, RegistroCNAB::LISTA_TIPO_REGISTROS_UNICO, $this->_nrOrg, $this->_cdBanco, $this->_nrTipoMovimento, $this->_cdOperador
                );

                $dtPagamento = $this->_idUtilDtPgtoItem==='S' ? $this->_dtPagamento : null;

                $vinculos = $this->_vinculom->getVinculosAtivosParaEvento(
                    $this->_nrOrg, $this->_mesCompetencia, $this->_dsInicio, $this->_dsFim, $this->_nrEvento,
                    $this->_nrTipoMovimento, $this->_dsOrdem, $dtPagamento, $this->_tpVinculo
                );

            }
            catch(Exception $e){
                throw new \Exception("Erro ao inicializar preparação do CNAB");
            }

            $idstatus = MsgCalculo::ID_PENDENTE;
            $nrcalculomsg = $this->_nrCalculoFolha;
            
            foreach ($vinculos as $vinculo) {

                $processamento = MsgCalculo::getInstance();
                $processamento->setDados($nrcalculomsg, $vinculo->getNrvinculom(), $this->_nrOrg, "CNAB");
                $this->_cnab->preparaCNAB($vinculo);
                
                $msgcalculo = $processamento->getDados();

                if ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ERRO) {
                    $idstatus = MsgCalculo::ID_COM_ERRO;
                } elseif ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ALERTA and $idstatus != MsgCalculo::ID_COM_ERRO) {
                    $idstatus = MsgCalculo::ID_COM_ALERTA;
                }


                if ($msgcalculo['IDABORT'] === true) {
                    $idabort = true;
                    break;
                } else {
                    $processamento->salvaMensagens();
                }

                $processamento->clear();			
            }

            $qtdRegistros = count($this->_cnabMagnetico->getItems());            
            

            if ($qtdRegistros > 0) {
                $persisteCnab = $this->_cnabMagnetico->persist();
                if ($persisteCnab === true) {
                    $this->_msgSucessoPersisteMagnetico = "CNAB gravado com sucesso!";
                }
            }
        }else{
            throw new \Exception('Selecione para processar o vinculo');
        }
        
        if (($idabort == false) and ($idstatus !== MsgCalculo::ID_COM_ERRO)) {
            // Gera o magnetico.
            if ($this->_idGeraMeioMagnetico === 'S') {
                try {
                    $nmArquivo = $this->_geraMagnetico();
                } catch (Exception $e) {
                    throw new \Exception("Erro ao gerar magnético." . $e->getMessage());
                }
            } else {
                throw new \Exception('Selecione para gerar o magnetico');
            }
        }

        return array(
            'ARQUIVO' => $nmArquivo, 
            'CALCULO' => $nrcalculomsg,
            'MENSAGENS' => ($idabort === true ? $msgcalculo['MENSAGENS'] : null),
            'ERRO' => (($idstatus === MsgCalculo::ID_COM_ERRO) or ($idstatus === MsgCalculo::ID_COM_ALERTA)),			
            'ABORT' => $idabort
        );
    }

    private function _geraCNAB_Agrup(){

        $idabort = false;
        $agrupCnab = array();

        if ($this->_idProcessaVinculo === 'S') {
            try {
                $dtPagamento = $this->_idUtilDtPgtoItem === 'S' ? $this->_dtPagamento : null;

                $vinculos = $this->_vinculom->getVinculosAtivosParaEvento(
                    $this->_nrOrg, $this->_mesCompetencia, $this->_dsInicio, $this->_dsFim, $this->_nrEvento,
                    $this->_nrTipoMovimento, $this->_dsOrdem, $dtPagamento, $this->_tpVinculo
                );

                foreach ($vinculos as $vinculo) {
                    if ($this->_idAgrupArquivo == 'ESTRUTGEREN') {
                        $agrupCnab[$vinculo->historico->getNrestrutgeren()][] = $vinculo;
                    }
                    if ($this->_idAgrupArquivo == 'VINCULO') {
                        $agrupCnab[$vinculo->getNrvinculom()][] = $vinculo;
                    }
                }
            } catch (Exception $e) {
                throw new \Exception("Erro ao inicializar preparação do CNAB");
                throw $e;
            }
        }else{
            throw new \Exception('Selecione para processar o vinculo');
        }

        $idstatus = MsgCalculo::ID_PENDENTE;
        $nrcalculomsg = $this->_nrCalculoFolha;

        foreach ($agrupCnab as $idAgrup => $agrup) {

            $this->_cnabMagnetico->inicializaListaRegistros(
                $this->_mesCompetencia, RegistroCNAB::LISTA_TIPO_REGISTROS_UNICO, $this->_nrOrg, $this->_cdBanco, $this->_nrTipoMovimento, $this->_cdOperador
            );

            $this->_cnab->setValorTotalPagamento(0);

            foreach ($agrup as $vinculo) {
                $processamento = MsgCalculo::getInstance();
                $processamento->setDados($nrcalculomsg, $vinculo->getNrvinculom(), $this->_nrOrg, "CNAB");

                $this->_cnab->preparaCNAB($vinculo);

                $msgcalculo = $processamento->getDados();

                if ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ERRO) {
                    $idstatus = MsgCalculo::ID_COM_ERRO;
                } elseif ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ALERTA and $idstatus != MsgCalculo::ID_COM_ERRO) {
                    $idstatus = MsgCalculo::ID_COM_ALERTA;
                }

                if ($msgcalculo['IDABORT'] === true) {
                    $idabort = true;
                    break;
                } else {
                    $processamento->salvaMensagens();
                }

                $processamento->clear();
            }

            $qtdRegistros = count($this->_cnabMagnetico->getItems());

            if ($qtdRegistros > 0) {
                $persisteCnab = $this->_cnabMagnetico->persist();
                if ($persisteCnab === true) {
                    $this->_msgSucessoPersisteMagnetico = "CNAB gravado com sucesso!";
                }
            }

            if (($idabort == false) and ($idstatus !== MsgCalculo::ID_COM_ERRO)) {
                if ($this->_idGeraMeioMagnetico === true) {
                    try {
                        $nmArquivo = $this->_geraMagnetico($idAgrup);
                    } catch (Exception $e) {
                        throw new \Exception("Erro ao gerar magnético." . $e->getMessage());
                    }
                } else {
                    throw new \Exception('Selecione para gerar o magnetico');
                }
            }

            $this->_cnabMagnetico->resetItems();
            $this->_cnabMagnetico->retroceder($this->_nrOrg, $this->_mesCompetencia, $this->_cdBanco, $this->_nrTipoMovimento, $this->_cdOperador);
        }

        return array(
            'ARQUIVO' => $nmArquivo,
            'CALCULO' => $nrcalculomsg,
            'MENSAGENS' => ($idabort === true ? $msgcalculo['MENSAGENS'] : null),
            'ERRO' => (($idstatus === MsgCalculo::ID_COM_ERRO) or ($idstatus === MsgCalculo::ID_COM_ALERTA)),
            'ABORT' => $idabort
        );
    }

    private function _geraMagnetico($idArquivo = null){
        
        $resultado = $this->_cnabMagnetico->retornaRegistros($this->_mesCompetencia, $this->_nrOrg, $this->_cdBanco, $this->_nrTipoMovimento, $this->_cdOperador);
        
        if (count($resultado) > 0) {
            $this->_constroiNomeArquivo($idArquivo);
            
            $nomeArquivo = $this->_nomeArquivo;
            
            $this->_contador = 1;
            
            foreach ($resultado as $item) {

                $linhaRegistro = $this->_modificaLinhas($item);

                if ($item->getIdtiporegistro() === Registro5CNABSantander::REGISTRO_5) {
                    $valorTotalPagamento = $this->_cnab->getValorTotalPagamento();
                    $item->setValor($valorTotalPagamento);
                }

                $this->entityManager->persist($item);
                $this->entityManager->flush($item);
    
                FPA::escreveArquivo($nomeArquivo, $linhaRegistro);
                
                if (end($resultado) !== $item) {
                    FPA::escreveArquivo($nomeArquivo, RegistroCNAB::QUEBRA_LINHA_CR_LF);
                }
            }

            if (empty($geraArquivo) === false) {
                $this->_msgSucessoGeraMagnetico =  "Arquivo gerado com sucesso!";
            }
            return $nomeArquivo;
        }else{
            throw new \Exception('Não existe registro processado para o vinculo ');
        }
    }

    private function _modificaLinhas($itemCnabMagnetico) {
        $linhaRegistro = $itemCnabMagnetico->getDslinharegistro();
        $tipoRegistro = $itemCnabMagnetico->getIdtiporegistro();

        if ($tipoRegistro === RegistroCNAB::REGISTRO_3) {
            $linhaRegistro = $this->_cnab->modificaSequencia($linhaRegistro, $this->_contador);
            $itemCnabMagnetico->setDslinharegistro($linhaRegistro);
            $this->_contador++;
        } else {
            $linhaRegistro = $this->_cnab->modificaLinhasConsolidacao($tipoRegistro, $linhaRegistro);
            $itemCnabMagnetico->setDslinharegistro($linhaRegistro);
        }

        return $linhaRegistro;
    }

    private function _constroiNomeArquivo($idArquivo = '') {
        /*$ultimoSequencial = $this->_cnabMagnetico->retornaUltimoSequencialRegistro0($this->_mesCompetencia,
            $this->_nrOrg, $this->_cdBanco, $this->_cdOperador);

        if (empty($ultimoSequencial) === false) {
            $sequencia = $ultimoSequencial;
            $sequencia++;
        } else {
            $sequencia = FPARegistro::CARACTER_ONZE;
        }*/
        $sequencia = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'CNABBB', 1, 12, true);
        
        $this->_nomeArquivo = 'PG'.(string)$sequencia.$this->_mesCompetencia->format('m').$idArquivo.self::EXTENSAO_ARQUIVO;
    }

    private function _buscaDadosFormulario($dataSet){
        $this->_dsInicio = $dataSet['DSINICIO'];
        $this->_dsFim = $dataSet['DSFIM'];
        $this->_dsOrdem = $dataSet['DSORDEM'];
        $this->_nrTpHierarquiaOrg = $dataSet['NRHIERARQORG'];
        $this->_idProcessaVinculo = $dataSet['IDPROCVINCULO'];
        $this->_idGeraMeioMagnetico = $dataSet['IDGERAMEIOMAG'];
        $this->_filePath = $dataSet['DSCAMINHOARQUIVO'];
        $this->_nrEstruturaResp = $dataSet['NRESTRUTRESPONS'];
        $this->_tpInscResp = $dataSet['IDTPINSCRESPONS'];
        $this->_nrEvento = $dataSet['NREVENTOM'];
        $this->_nrTipoMovimento = $dataSet['NRTIPOMOVIMENT'];
        $this->_cdBanco = $dataSet['CDBANCO'];
        $this->_nrGrupoCnab = $dataSet['NRGRUPOCNAB'];
        $this->_dtPagamento = $dataSet['DTPAGAMENTO'];
        $this->_idUtilDtPgtoItem = $dataSet['IDUTILDTPGTOITEM'];
        $this->_idIgnoraOcorrencia = $dataSet['IDIGNORAOCORRENCIA'];
        $this->_idAgrupArquivo = $dataSet['IDAGRUPARQUIVO'];
    }
}