<?php
/**
 * Created by PhpStorm.
 * User: matheussilva
 * Date: 27/10/14
 * Time: 16:56
 */

namespace HCM\FPA\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\Repositories;
use HCM\FPA\ExportacaoExcelService;


class ExportacaoExcelRepository {
    
    const ERRO  = 1111;
    const AVISO = 1110;

    const MSG_FALTA_VINCULO          = '';
    const MSG_NAO_EXISTE_ESTRUT      = 'Não existe a Estrutura informada';
    const MSG_NENHUMA_ESTRUTURA      = 'Nenhuma Estrutura foi informado';
    const MSG_ESTRUTURA_HIERARQUIACA = 'Quando existir uma Estrutura hierárquica selecionada um Nome de Hierarquia também deverá ser informado';

    const NMFERIAS = 'Férias';

    /**
     * @param int $nrestrutlegal
     * @param int $nrestrutgeren
     * @param int $nrestrutsind
     * @param array $eventos
     * @param String $dtmescompetencia
     * @param int $nrorg
     *
     * @throws \Exception
     */
    public function __construct(Connection $connection, EntityManager $entityManager) {
        $this->connection    = $connection;
        $this->entityManager = $entityManager;
    }

    /**
     * CSU003 - Verifica a integridade da informação.
     *
     * @param int $nrestrutlegal
     * @param int $nrestrutgeren
     * @param int $nrestrutsind
     * @param array $eventos
     * @param int $nrorg
     *
     * @return array
     * @throws \Exception
     */
    private function verificarInformacoes( $nrestrutlegal, $nrestrutgeren, $nrestrutsind, $nrtipohierarq, $nrestruthierarq, $eventos, $nrorg)
    {
        try {
            if (!empty($nrestrutlegal) || !empty($nrestrutgeren) || !empty($nrestrutsind) || !empty($nrtipohierarq) || !empty($nrestruthierarq)) {
                $this->verificaEstrutura($nrestrutlegal, $nrorg);
                $this->verificaEstrutura($nrestrutgeren, $nrorg);
                $this->verificaEstrutura($nrestrutsind, $nrorg);
                $cjEventos = $this->verificaEventos($eventos, $nrorg);
                if(!empty($nrestruthierarq) && empty($nrtipohierarq)){
                    throw new \Exception(self::MSG_ESTRUTURA_HIERARQUIACA);
                }
            } else {
                throw new \Exception(self::MSG_NENHUMA_ESTRUTURA);
            }

            if ($cjEventos) {
                return $cjEventos;
            } else {
                return false;
            }
        } catch (\Exception $e) {
            if($e->getCode() == self::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * CSU003a - Verifica se a estrutura existe.
     *
     * @param $nrestrutura
     * @param $nrorg
     *
     * @return boolean
     * @throws \Exception
     */
    private function verificaEstrutura( $nrestrutura, $nrorg)
    {
        try {
            if ($nrestrutura) {
                $params = array('P_NRORG' => $nrorg,
                    'P_NRESTRUTURAM' => $nrestrutura);
                $where = 'NRORG = :P_NRORG AND NRESTRUTURAM = :P_NRESTRUTURAM';
                $object = Estruturam::getObjectByWhere($where, $params);
            }else{
                return false;
            }

            if (!$object) {
                throw new \Exception(self::MSG_NAO_EXISTE_ESTRUT, self::ERRO);
            }else{
                return true;
            }
        } catch (\Exception $e) {
            if($e->getCode() == self::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * @todo CSU003b - Verifica se a estrutura gerencial pertence a estrutura legal
     *
     * @param int $nrestrutlegal
     * @param int $nrestrutgeren
     * @param int $nrorg
     *
     * @throws \Exception
     */
    private function verificaRelacaoEstrutura( $nrestrutlegal, $nrestrutgeren, $nrorg)
    {}

    /**
     * CSU003c - Verifica a consistência dos eventos capturados.
     * @param array $eventos
     *
     * @return array
     * @throws \Exception
     */
    private function verificaEventos( $eventos)
    {
        if(!is_array($eventos))
            throw new \Exception('Nenhum evento selecionado.');
        try {
            $cjeventos = array();
            foreach ($eventos as $evento) {
                $cjeventos[] = array('CODIGO' => $evento['nreventom'],
                                     'EVENTO' => $evento['nmeventoh']);
            }
            return $cjeventos;
        } catch (\Exception $e) {
            if($e->getCode() == self::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * CSU004 - Busca os vínculos
     *
     * @param int $nrestrutlegal
     * @param int $nrestrutgeren
     * @param int $nrestrutsind
     * @param String $dtmescompetencia
     * @param int $nrorg
     *
     * @return array
     * @throws \Exception
     */
    public function getVinculos( $row){
        //$info['NRESTRUTLEGAL'], $info['NRESTRUTGEREN'], $info['NRESTRUTSIND'], $info['DTMESCOMPENTEC'], $info['NRORGTRAB'], $info['NRTIPOHIERARQORG'], $info['NRESTRUTURAM']);
            
        try {
            $where = array();
            if (!empty($row['NRESTRUTLEGAL'])) {
                $where[] = 'VH.NRESTRUTLEGAL IN (' . implode(',', $row['NRESTRUTLEGAL']) . ')';
            }
            if (!empty($row['NRESTRUTGEREN'])) {
                $where[] = 'VH.NRESTRUTGEREN IN (' . implode(',', $row['NRESTRUTGEREN']) . ')';
            }
            if (!empty($row['NRESTRUTSIND'])) {
                $where[] = 'VH.NRESTRUTSIND IN (' . implode(',', $row['NRESTRUTSIND']) . ')';
            }
            if (!empty($row['NRESTRUTURAM']) && !empty($row['NRTIPOHIERARQORG']) ) {
                $where[] = 'EHA.NRESTRUTURAM = ' . $row['NRESTRUTURAM'];
            }

            if(empty($where)) { //Se não houver parâmetros de estruturas não se deve exeutar o implode para não gerar um 'AND ...' orfão.
                $where = '';
            }else{
                $where = implode(' AND ', $where);
                $where = " AND $where";
            }
            
            $objs = $this->getVinculosH($where, $row);
            if($objs){
                return $objs->fetchAll();
            } else {
                return false;    
            }
        } catch (\Exception $e) {
            if($e->getCode() == self::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * CSU004a - Busca vínculos relacionados ao filtros de estruturas
     *
     * @param String $where
     * @param String $dtmescompetencia
     * @param int $nrorg
     *
     * @return bool|GpeVinculoh
     * @throws \Exception
     */
    private function getVinculosH($where, $row){
        //$dtmescompetencia, $nrorg, $nrtipohierarq, $nrestruthierarq
        try {
            $params = array(':P_NRORG'           => $row['NRORGTRAB'],
                            ':P_COMPETENCIA'     => $row['DTMESCOMPENTEC'],
                            ':P_NRTIPOHIERARQ'   => $row['NRTIPOHIERARQORG']);
            $sql_1 = "SELECT * FROM GPE_VINCULOH VH JOIN GPE_VINCULOM VM ON VM.NRORG = VH.NRORG AND VM.NRVINCULOM = VH.NRVINCULOM AND ( VM.DTRESCISAOVINC >= LAST_DAY(:P_COMPETENCIA) OR VM.DTRESCISAOVINC IS NULL )";
            $sql_h = "JOIN ESTRUTURAH EHA ON EHA.NRORG = VH.NRORG AND EHA.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH WHERE DTMESCOMPETENC <= :P_COMPETENCIA AND NRORG = VH.NRORG AND NRESTRUTURAM = EHA.NRESTRUTURAM ) AND EHA.NRESTRUTURAM = ( SELECT MAX(H.NRESTRUTURAMSUP) FROM HIERARQUIAORG H WHERE H.NRORG = VH.NRORG AND H.NRESTRUTURAMREL = VH.NRESTRUTGEREN AND H.NRTIPOHIERARQORG = :P_NRTIPOHIERARQ AND H.DTINIVIGENCIA = ( SELECT MAX(DTINIVIGENCIA) FROM HIERARQUIAORG WHERE DTINIVIGENCIA <= LAST_DAY(:P_COMPETENCIA) AND NRORG = VH.NRORG AND NRESTRUTURAMREL = VH.NRESTRUTGEREN AND NRTIPOHIERARQORG = H.NRTIPOHIERARQORG ) )";
            $sql_2 = "WHERE VH.NRORG = :P_NRORG AND VH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH WHERE DTMESCOMPETENC <= :P_COMPETENCIA AND NRORG = VH.NRORG AND NRVINCULOM = VH.NRVINCULOM )";
            $sql_3 = "ORDER BY VH.NRESTRUTLEGAL/*, VH.NRESTRUTGEREN*/, VM.NMVINCULOM";
            $sql_est= "LEFT JOIN ESTRUTURAM EMG ON EMG.NRORG        = VM.NRORG AND EMG.NRESTRUTURAM = VH.NRESTRUTGEREN LEFT JOIN ESTRUTURAH EHG ON EHG.NRORG        = VM.NRORG AND EHG.NRESTRUTURAM = EMG.NRESTRUTURAM AND EHG.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH WHERE DTMESCOMPETENC <= :P_COMPETENCIA AND NRORG        = VM.NRORG AND NRESTRUTURAM = EMG.NRESTRUTURAM )";
            $sql_4 = "LEFT JOIN GPE_OCUPACAOH OH ON OH.NRORG = VM.NRORG AND OH.NROCUPACAOM = NVL( VH.NRCARGO, VH.NRFUNCAO ) AND OH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_OCUPACAOH WHERE DTMESCOMPETENC <= :P_COMPETENCIA AND NRORG = OH.NRORG AND NROCUPACAOM = OH.NROCUPACAOM )";
            $sql_5 = "LEFT JOIN GPE_ALTESITUFUNC ALTF ON ALTF.NRORG      = VM.NRORG AND ALTF.NRVINCULOM = VM.NRVINCULOM AND ALTF.DTINISITUFUNC = ( SELECT MAX(DTINISITUFUNC) FROM GPE_ALTESITUFUNC WHERE DTINISITUFUNC <= LAST_DAY(:P_COMPETENCIA) AND NRORG      = ALTF.NRORG AND NRVINCULOM = ALTF.NRVINCULOM )LEFT JOIN GPE_SITUFUNCH SFH ON ( SFH.NRORG IS NULL OR SFH.NRORG = 0 OR SFH.NRORG = VM.NRORG ) AND SFH.NRSITUFUNCM = ALTF.NRSITUFUNCM /*VH.NRSITUFUNCM*/ AND SFH.DTEFETIVA = ( SELECT MAX(DTEFETIVA) FROM GPE_SITUFUNCH WHERE DTEFETIVA <= :P_COMPETENCIA AND ( SFH.NRORG IS NULL OR SFH.NRORG = 0 OR SFH.NRORG = VM.NRORG ) AND NRSITUFUNCM = SFH.NRSITUFUNCM )";
            if(!empty($row['NRTIPOHIERARQORG'])){    //Rafael Pickbrenner PCK 25/09/2015 14121
                $sql = "$sql_1 $sql_est $sql_h  $sql_4 $sql_5 $sql_2 $where $sql_3";
            }else{
                //$where = "$where AND NRORG = :P_NRORG AND NRVINCULOH IN (SELECT NRVINCULOH FROM GPE_VINCULOH INNER JOIN GPE_VINCULOM ON GPE_VINCULOM.NRORG = GPE_VINCULOH.NRORG AND GPE_VINCULOM.NRVINCULOM = GPE_VINCULOH.NRVINCULOM WHERE TRUNC(GPE_VINCULOM.DTADMISSAOVINC, 'DD') <= :P_DTMESCOMPETENC AND (TRUNC(GPE_VINCULOM.DTRESCISAOVINC, 'DD') >= :P_DTMESCOMPETENC OR GPE_VINCULOM.DTRESCISAOVINC IS NULL) AND DTMESCOMPETENC = (SELECT MAX(MAX_VINCULOH.DTMESCOMPETENC) FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG AND MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND TRUNC(MAX_VINCULOH.DTMESCOMPETENC, 'DD') <= :P_DTMESCOMPETENC AND MAX_VINCULOH.NRORG = :P_NRORG))";
                $sql = "$sql_1 $sql_est $sql_4 $sql_5 $sql_2 $where $sql_3";
            }
            //$object = GpeVinculoh::getObjectsByWhere($where, $params, 'NRESTRUTGEREN, NRVINCULOM'); //Original, tipo objeto
            $query = $this->entityManager->getConnection()->prepare($sql);
            
            if(!empty($row['NRTIPOHIERARQORG'])){
                $query->execute([
                  "P_COMPETENCIA"    => $row['DTMESCOMPENTEC'],
                  "P_NRORG"          => $row['NRORGTRAB'],
                  "P_NRTIPOHIERARQ"  => $row['NRTIPOHIERARQORG']
                ]
            );
            }else{
                $query->execute([
                  "P_COMPETENCIA"    => $row['DTMESCOMPENTEC'],
                  "P_NRORG"          => $row['NRORGTRAB']
                ]
            );
            }
            if ($query) {
                return $query;
            } else {
                return false;
            }
        } catch (\Exception $e) {
            if($e->getCode() == self::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * CSU005 - Busca informações do vínculo
     *
     * @param array $infoVinculos
     * @param String $dtmescompetencia
     * @param int $nrorg
     *
     * @return array
     * @throws \Exception
     */
    private function getInfoVinculos( $infoVinculos, $dtmescompetencia, $nrorg, $nrtipohierarq )
    {
        /**
         * @var GpeVinculoh $infoVinculo
         * @var GpeVinculom $objVinculom
         */
        try {
            $info = array();
            $nrestrutgeren = -1;

            foreach ($infoVinculos as $infoVinculo) {
                if(isset($infoVinculo['NRESTRUTURAM'])){ //Verifica se deve utilizar a estrutura hierárquica ou gerencial
                    $nrestrutura = (int)$infoVinculo['NRESTRUTURAM'];
                    if($nrestrutura > 0){
                        $nmhierarq = \Tipohierarqorg::getObjectsByWhere("(NRORG = 0 OR NRORG = $nrorg) AND NRTIPOHIERARQORG = $nrtipohierarq");
                        $nmhierarq = $nmhierarq[0]->getNmtipohierarqorg();
                    }else{
                        $nmhierarq = 'Hierarquia não cadastrada';
                    }
                }else{
                    $nrestrutura = $infoVinculo['NRESTRUTGEREN'];
                    $nmhierarq   = 'Estrutura Gerencial';
                }
                if ($nrestrutgeren != $nrestrutura) {
                    $nrestrutgeren = $nrestrutura;
                    if (!$info[$nrestrutgeren]) {
                        $info[$nrestrutgeren] = array();
                    }
                }

                //Objeto Vínculom
                //$objVinculom = $infoVinculo->getVinculom(); //Original
                $objVinculom = GpeVinculom::findOneByNrvinculomAndNrorg($infoVinculo['NRVINCULOM'], $infoVinculo['NRORG']);

                //Ferias e Situação Funcional
                if (!$this->getFerias($infoVinculo['NRVINCULOM'], $nrorg, $dtmescompetencia)) {
                    $nmsitufunc = $this->getSitufunc($infoVinculo['NRVINCULOM'], $nrorg, $dtmescompetencia);
                } else {
                    $nmsitufunc = self::NMFERIAS;
                }

                //Cargo
                $nmcargo = $this->getCargo($infoVinculo['NRVINCULOM'], $nrorg, $dtmescompetencia);

                //Construir Informação
                $info[$nrestrutgeren][] = array('ID' => $infoVinculo['NRVINCULOM'],
                    'NOME' => $objVinculom->getNmvinculom(),
                    'SITUFUNC' => $nmsitufunc,
                    'ADMISSAO' => $objVinculom->getDtadmissaovinc()->format('d/m/Y'),
                    'CARGO' => $nmcargo,
                    'ESTRUTLEGAL' => $this->getNmestrutura($infoVinculo['NRESTRUTLEGAL'], $nrorg, $dtmescompetencia),
                    'ESTRUTGEREN' => $nrestrutura > 0 ? $this->getNmestrutura($nrestrutura, $nrorg, $dtmescompetencia) : 'sem estrutura',
                    'ESTRUTSINDI' => $this->getNmestrutura($infoVinculo['NRESTRUTSIND'],  $nrorg, $dtmescompetencia),
                    'COMPETENCIA' => $dtmescompetencia,
                    'NMESTRUTURA' => $nmhierarq,
                    'NMGEREN'     => $this->getNmestrutura($infoVinculo['NRESTRUTGEREN'], $nrorg, $dtmescompetencia) );


            }

            return $info;
        } catch (\Exception $e) {
            if($e->getCode() == self::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * CSU005a - Verifica se há um objeto de FpaGozoferias
     *
     * @param int $nrvinculom
     * @param int $nrorg
     * @param String $dtmescompetencia
     *
     * @return GpeVinculom|bool
     * @throws \Exception
     */
    private function getFerias( $nrvinculom, $nrorg, $dtmescompetencia)
    {
        try {
            $params = array('P_NRORG' => $nrorg,
                'P_NRVINCULOM' => $nrvinculom,
                'P_DTMESCOMPETENCIA' => $dtmescompetencia);
            $where = "NRORG = :P_NRORG AND NRGOZOFERIAS IN (SELECT NRGOZOFERIAS FROM FPA_GOZOFERIAS WHERE NRORG = :P_NRORG AND DTINIGOZOFERIAS = (SELECT MAX(MAX_GOZOFERIAS.DTINIGOZOFERIAS) FROM FPA_GOZOFERIAS MAX_GOZOFERIAS WHERE MAX_GOZOFERIAS.NRORG = FPA_GOZOFERIAS.NRORG AND MAX_GOZOFERIAS.NRFERIAS = FPA_GOZOFERIAS.NRFERIAS AND(MAX_GOZOFERIAS.DTINIGOZOFERIAS BETWEEN TRUNC(TO_DATE(:P_DTMESCOMPETENCIA),'MONTH') AND LAST_DAY(TO_DATE(:P_DTMESCOMPETENCIA)) OR MAX_GOZOFERIAS.DTFIMGOZOFERIAS BETWEEN TRUNC(TO_DATE(:P_DTMESCOMPETENCIA),'MONTH') AND LAST_DAY(TO_DATE(:P_DTMESCOMPETENCIA))) AND NRFERIAS IN (SELECT NRFERIAS FROM FPA_FERIAS WHERE NRORG = :P_NRORG AND NRVINCULOM = :P_NRVINCULOM )))";
            $result = FpaGozoferias::getObjectsByWhere($where, $params);
            if ($result) {
                return true;
            } else {
                return false;
            }
        } catch (\Exception $e) {
            if($e->getCode() == self::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * CSU005b - Busca objeto de Gpe
     *
     * @param int $nrvinculom
     * @param int $nrorg
     * @param String $dtmescompetencia
     *
     * @return GpeVinculom|bool
     * @throws \Exception
     */
    private function getSitufunc( $nrvinculom, $nrorg, $dtmescompetencia){
        /**
         * @var GpeAltesitufunc $objAltesitufunc
         * @var GpeSitufunch $objSitufunc
         */
        try {
            $params = array('P_NRORG' => $nrorg,
                'P_NRVINCULOM' => $nrvinculom,
                'P_DTMESCOMPETENCIA' => $dtmescompetencia);
            $where = "NRORG = :P_NRORG AND NRVINCULOM = :P_NRVINCULOM AND NRALTESITUFUNC = (SELECT MAX(NRALTESITUFUNC) FROM GPE_ALTESITUFUNC WHERE GPE_ALTESITUFUNC.NRORG = :P_NRORG AND NRVINCULOM = :P_NRVINCULOM AND DTINISITUFUNC = (SELECT MAX(MAX_ALTESITUFUNC.DTINISITUFUNC) FROM GPE_ALTESITUFUNC MAX_ALTESITUFUNC WHERE MAX_ALTESITUFUNC.DTINISITUFUNC <= :P_DTMESCOMPETENCIA AND MAX_ALTESITUFUNC.NRORG = GPE_ALTESITUFUNC.NRORG AND MAX_ALTESITUFUNC.NRVINCULOM = GPE_ALTESITUFUNC.NRVINCULOM ))";
            $objAltesitufunc = GpeAltesitufunc::getObjectByWhere($where, $params);

            if ($objAltesitufunc) {
                $params = array('P_NRSITUFUNCM' => $objAltesitufunc->getNrsitufuncm());
                $where = "NRSITUFUNCH IN ( SELECT NRSITUFUNCH FROM GPE_SITUFUNCH WHERE NRSITUFUNCM = :P_NRSITUFUNCM AND DTEFETIVA = (SELECT MAX(MAX_SITUFUNCH.DTEFETIVA) FROM GPE_SITUFUNCH MAX_SITUFUNCH WHERE MAX_SITUFUNCH.NRSITUFUNCM = GPE_SITUFUNCH.NRSITUFUNCM AND MAX_SITUFUNCH.NRSITUFUNCM = :P_NRSITUFUNCM))";
                $objSitufunc = GpeSitufunch::getObjectByWhere($where, $params);
            }

            if ($objSitufunc) {
                return $objSitufunc->getNmsitufunch();
            } else {
                return '';
            }
        } catch (\Exception $e) {
            if($e->getCode() == self::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * CSU005c - Busca objeto de FpaGozoferias
     *
     * @param int $nrvinculom
     * @param int $nrorg
     * @param String $dtmescompetencia
     *
     * @return GpeVinculom|bool
     * @throws \Exception
     */
    public function getCargo( $nrvinculom, $nrorg, $dtmescompetencia){
        /**
         * @var GpeAlteocupacao $objAlteocup
         * @var GpeOcupacaoh $objOcupacao
         */
        try {
            $params = array('P_NRVINCULOM' => $nrvinculom,
                'P_NRORG' => $nrorg,
                'P_DTMESCOMPETENCIA' => $dtmescompetencia);
            $where = "  NRORG = :P_NRORG  AND NRALTEOCUPACAO IN ( SELECT NRALTEOCUPACAO FROM GPE_ALTEOCUPACAO WHERE NRORG = :P_NRORG AND NRVINCULOM = :P_NRVINCULOM AND DTINIOCUPACAO = (SELECT MAX(MAX_ALTEOCUPACAO.DTINIOCUPACAO) FROM GPE_ALTEOCUPACAO MAX_ALTEOCUPACAO WHERE MAX_ALTEOCUPACAO.NRVINCULOM = GPE_ALTEOCUPACAO.NRVINCULOM AND MAX_ALTEOCUPACAO.NRORG = GPE_ALTEOCUPACAO.NRORG AND TRUNC(MAX_ALTEOCUPACAO.DTINIOCUPACAO, 'DD') <= :P_DTMESCOMPETENCIA ) )";
            return $where;
        } catch (\Exception $e) {
            if($e->getCode() == self::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }

    }

    /**
     * CSU005d - Busca objeto de GpeVinculom
     *
     * @param int $nrvinculom
     * @param int $nrorg
     *
     * @return bool
     */
    private function getVinculom( $nrvinculom, $nrorg)
    {
        try {
            $param = array('P_NRVINCULOM' => $nrvinculom,
                'P_NRORG' => $nrorg);
            $where = "NRORG = :P_NRORG AND NRVINCULOM = :P_NRVINCULOM";
            $objVinculom = GpeVinculom::getObjectByWhere($where, $param);

            if ($objVinculom) {
                return $objVinculom;
            } else {
                return false;
            }
        } catch (\Exception $e) {
            if($e->getCode() == self::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    private function getNmestrutura( $nrestruturam, $nrorg, $dtmescompetencia)
    {
        /**
         * @var Estruturah $objEstruturam
         */
        try {
            $params = array('P_NRESTRUTURAM' => $nrestruturam,
                            'P_NRORG' => $nrorg,
                            'P_DTMESCOMPETENCIA' => $dtmescompetencia);
            $where = " NRORG = :P_NRORG AND NRESTRUTURAH IN ( SELECT NRESTRUTURAH FROM ESTRUTURAH WHERE NRORG = :P_NRORG AND NRESTRUTURAM = :P_NRESTRUTURAM AND DTMESCOMPETENC = (SELECT MAX(MAX_ESTRUTURAH.DTMESCOMPETENC) FROM ESTRUTURAH MAX_ESTRUTURAH WHERE MAX_ESTRUTURAH.NRORG = ESTRUTURAH.NRORG AND MAX_ESTRUTURAH.NRESTRUTURAM = ESTRUTURAH.NRESTRUTURAM AND TRUNC(MAX_ESTRUTURAH.DTMESCOMPETENC, 'DD') <= :P_DTMESCOMPETENCIA))";
            $objEstruturam = Estruturah::getObjectByWhere($where, $params);

            if ($objEstruturam) {
                return $objEstruturam->getNmrazsocestrut();
            } else {
                return '';
            }
        } catch (\Exception $e) {
            if($e->getCode() == self::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }
}