<?php

namespace HCM\FPA\Services;


use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\FPA\Services\ExportacaoExcelService;
use Zeedhi\Framework\HTTP\Kernel;
use HCM\Util\FPA;

class LoteVinculoService {
    
    protected $entityManager;
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters,  ExportacaoExcelService $exportacaoExcelService, Kernel $httpKernel) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->exportacaoExcelService = $exportacaoExcelService;
        $this->httpKernel = $httpKernel;
        $this->itemlotevinc = Repositories::FPA_ITEMLOTEVINC;
    }

    public function importarPlanilhaLoteVinculo($row)
    {   
        //Dados básicos
        $nrorg       = $this->hcmParameters->getNrorgTrabalho();
        $nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
        $msgRetorno   = array(); //Mensages de erro, se estiver vazio não houve erros

        $dados = $this->getDadosPlanilhaLoteVinculo($row['DSPATH'][0]);
        foreach($dados as $dado) {
            $objVinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array('nrvinculom' => $dado, 'nrorg' =>  $nrorg));
            if($objVinculom !== null) {
                $nrvinculom = $objVinculom->getNrvinculom();
                $existeVinculoNoLote = $this->entityManager->getRepository(Repositories::FPA_ITEMLOTEVINC)->findOneBy(array('nrlotevinc' => $row['NRLOTEVINC'], 'nrorg' => $nrorg, 'nrvinculom' => $nrvinculom));
                if($existeVinculoNoLote) {
                    $msgRetorno[] = [
                        'NRVINCULOM'  => $nrvinculom,
                        'DSDESCRICAO'  => 'Vínculo já esta cadastrado no lote',
                        'IDTIPOMSG' =>  'ATENÇÃO'
                    ];
                    continue;
                }
                $this->insereVinculoLote($nrvinculom , $nrorg, $row['NRLOTEVINC']);
            } else {
                $msgRetorno[] = [
                    'NRVINCULOM'  => $dado,
                    'DSDESCRICAO'  => 'Vínculo não encontrado no sistema',
                    'IDTIPOMSG' =>  'ATENÇÃO'
                ];
                continue;
            }    
        }

        return $msgRetorno;

    }

    public function getDadosPlanilhaLoteVinculo($path)
    {
         $head = array(
            'NRVINCULOM',
            );
      

        $file = $this->httpKernel->factoryFromFileData($path, FPA::getTempDirectory()."/");
        $caminhoNovo = DIRECTORY_SEPARATOR . $file->getPathname();
        
        $file = \PHPExcel_IOFactory::load($caminhoNovo);
        $file->setActiveSheetIndex(0);
        
        return $this->exportacaoExcelService->preparaInfoLoteVinculo($file);
    }
    
    private function insereVinculoLote($nrvinculom, $nrorg, $nrlotevinc)
    {  
        $obj = new $this->itemlotevinc;
        $obj->setNrorg($this->hcmParameters->getNrorgTrabalho());
        $obj->setDtinclusao(DateUtil::getDataAtual());
        $obj->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $obj->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $obj->setDtultatu(DateUtil::getDataAtual());
        $obj->setCdoperultatu($this->hcmParameters->getCdoperador());
        $obj->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $obj->setIdativo('S');
        $obj->setNrlotevinc($nrlotevinc);
        $obj->setNrvinculom($nrvinculom);
        $obj->setNritemlotevinc(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ITEMLOTEVINC', 1, 10, true));
        $this->entityManager->persist($obj);
        $this->entityManager->flush();
    }

    public function baixarPlanilhaLoteVinculo($nrorg, $nmarquivo) {
        $cabecalho = array(
            "Nr. Vínculo",
        );
        $dadosExemplo[0] = array(
            "105800",
        );
        
        return $this->exportacaoExcelService->gravarTemplate_1($nrorg, $cabecalho, $nmarquivo, $dadosExemplo);
    }
}