<?php
namespace HCM\Geral\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\Repositories;
use HCM\Util\Util;
use HCM\Util\HCMParameters;
use HCM\Geral\Repository\DateTime;

class RateioLancamentoRepository {

    protected  $connection;
    public function __construct(Connection $connection, EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }

    public function getVinculosEstrutLegal(
                $nrorg,
                $dtmescompetenc,
        int     $nrestrutlegal,
                $nrestrutgeren = null

    ) { 

        if(is_array($nrestrutgeren)) {
            $condNrestrutgeren = "AND vh.nrestrutgeren IN (" . implode(",",$nrestrutgeren) . ")";
        } else {
            $condNrestrutgeren = null;
        }

        $gpevinculoh =  'Teknisa\Model\Gestaopessoas\Entities\GpeVinculoh';
        $gpevinculom =  'Teknisa\Model\Gestaopessoas\Entities\GpeVinculom';
    $sql = <<<SQL
        SELECT vm, vh
        FROM $gpevinculom vm
        JOIN $gpevinculoh vh WITH vh.nrvinculom = vm.nrvinculom
                        AND vh.nrorg = vm.nrorg
                        AND vh.dtmescompetenc = (SELECT MAX(vhh.dtmescompetenc)
                                                   FROM $gpevinculoh vhh
                                                  WHERE vhh.dtmescompetenc <= :p_dtmescompetenc
                                                    AND vhh.nrorg = vh.nrorg
                                                    AND vhh.nrvinculom = vh.nrvinculom)
        WHERE vm.nrorg = :p_nrorg
        AND (vm.dtrescisaovinc >= :p_dtmescompetenc OR vm.dtrescisaovinc IS NULL)
        AND (vh.nrestrutlegal = :p_nrestrutlegal or :p_nrestrutlegal IS NULL)
        $condNrestrutgeren

        
SQL;
        $params = array(
            'p_dtmescompetenc'      => $dtmescompetenc,
            'p_nrorg'               => $nrorg,
            'p_nrestrutlegal'       => $nrestrutlegal
        );
        $resultado = $this->entityManager->createQuery($sql)->execute($params);

        $vinculo = array();
        $vinculom = Repositories::GPE_VINCULOM;
        foreach ($resultado as $key => $vinc) {
            if($vinc instanceof $vinculom){
                $vinc->historico = $resultado[$key+1];
                $vinculo[] = $vinc;
            }    
        }
        return $vinculo;
    }

    public function getLancamentos($nrorg, $nrrateio, $dtinicio, $dtfim)
    {   
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        $sql = $this->entityManager->getConnection()->prepare("
        SELECT LANC.*
        FROM FPA_RATEIOLANC LANC
        WHERE NRRATEIO = :P_NRRATEIO 
        AND LANC.NRORG = :P_NRORG
        AND (FIRST_DAY(LANC.DTLANCAMENTO) = :P_DTMESCOMPETENC)
        AND (LANC.DTLANCAMENTO >= :P_DTINICIO OR :P_DTINICIO IS NULL)
        AND (LANC.DTLANCAMENTO <= :P_DTFIM OR :P_DTFIM IS NULL)");
        $sql->execute(
            array(
                "P_NRRATEIO"         => $nrrateio,
                "P_NRORG"            => $nrorg,
                "P_DTINICIO"         => $dtinicio,
                "P_DTFIM"            => $dtfim,
                "P_DTMESCOMPETENC"   => $dtmescompetenc
                
            )
        );

        return $sql->fetchAll();
    }

    public function getItensValoresRateio($nrorg, $nrrateio, $dtinicio, $dtfim)
    {   
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        $sql = $this->entityManager->getConnection()->prepare("
        SELECT NRVINCULOM, SUM(VRVALOR) VRVALOR
        FROM FPA_RATEIOVAL
        WHERE NRRATEIO = :P_NRRATEIO 
        AND  NRORG = :P_NRORG
        AND (FIRST_DAY(DTDIALANC) = :P_DTMESCOMPETENC)
        AND (DTDIALANC >= :P_DTINICIO OR :P_DTINICIO IS NULL)
        AND (DTDIALANC <= :P_DTFIM OR :P_DTFIM IS NULL)
        GROUP BY NRVINCULOM");
        $sql->execute(
            array(
                "P_NRRATEIO"         => $nrrateio,
                "P_NRORG"            => $nrorg,
                "P_DTINICIO"         => $dtinicio,
                "P_DTFIM"            => $dtfim,
                "P_DTMESCOMPETENC"   => $dtmescompetenc
                
            )
        );

        return $sql->fetchAll();
    }

    public function buscaRateioExistente($nrorg, $inicioVigencia, $nrestruturam, $nreventom) {
        $sql = $this->entityManager->getConnection()->prepare(
            "SELECT RAT.* 
            FROM FPA_RATEIO RAT
            WHERE (RAT.NRORG = :P_NRORG 
            AND RAT.DTINIVIGENCIA = :P_DTINIVIGENCIA
            AND RAT.NRESTRUTURAM = :P_NRESTRUTURAM
            AND RAT.NREVENTOM = :P_NREVENTOM
            )"
        );
        
        $sql->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_DTINIVIGENCIA' => $inicioVigencia,
                'P_NRESTRUTURAM' => $nrestruturam,
                'P_NREVENTOM' => $nreventom
            )
        );
        
        return $sql->fetch();
    }

    public function buscaRateioExistenteOcupacao($nrorg, $nrestruturam, $nrocupacaom, $nrrateio) {
        $sql = $this->entityManager->getConnection()->prepare(
            "SELECT RAT.*
            FROM FPA_RATEIOOCUP RAT 
            WHERE (RAT.NRORG = :P_NRORG
            AND RAT.NRRATEIO = :P_NRRATEIO
            AND  RAT.NRESTRUTURAM = :P_NRESTRUTURAM
            AND RAT.NROCUPACAOM = :P_NROCUPACAOM)"
        );
        $sql->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_NRESTRUTURAM' => $nrestruturam,
                'P_NROCUPACAOM' => $nrocupacaom,
                'P_NRRATEIO' => $nrrateio
            )
        );
        
        return $sql->fetch();
    }

    public function buscaCalculoRateioExistente($nrorg, $nrrateio, $nrvinculom, $dtdialanc) {
        $sql = $this->entityManager->getConnection()->prepare(
            "SELECT RAT.*
            FROM FPA_RATEIOVAL RAT 
            WHERE (RAT.NRORG = :P_NRORG
            AND RAT.NRRATEIO = :P_NRRATEIO
            AND  RAT.NRVINCULOM = :P_NRVINCULOM
            AND RAT.DTDIALANC = :P_DTDIALANC)"
        );
        $sql->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_DTDIALANC' => $dtdialanc,
                'P_NRVINCULOM' => $nrvinculom,
                'P_NRRATEIO' => $nrrateio
            )
        );
        
        return $sql->fetch();
    }

    public function buscaRateioExistenteEstrutura($nrorg, $nrestruturam, $nrrateio) {
        $sql = $this->entityManager->getConnection()->prepare(
            "SELECT RAT.*
            FROM FPA_RATEIOESTRUT RAT 
            WHERE (RAT.NRORG = :P_NRORG
            AND RAT.NRRATEIO = :P_NRRATEIO
            AND    RAT.NRESTRUTURAM = :P_NRESTRUTURAM)"
        );
        $sql->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_NRESTRUTURAM' => $nrestruturam,
                'P_NRRATEIO' => $nrrateio
            )
        );
        
        return $sql->fetch();
    }

    public function buscaRateioExistenteLancamento($nrorg, $nrrateio, $dtlancamento) {
        $sql = $this->entityManager->getConnection()->prepare(
        "SELECT * FROM FPA_RATEIOLANC RAT WHERE (NRORG = :P_NRORG AND RAT.NRRATEIO = :P_NRRATEIO AND RAT.DTLANCAMENTO = :P_DTLANCAMENTO)"
        );
        
        $sql->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_NRRATEIO' => $nrrateio,
                'P_DTLANCAMENTO' => $dtlancamento
            )
        );
        
        return $sql->fetch();
    }
    
    public function selectExisteItensRateio($nrorg, $nrrateio, $tabela) {
        $sql = $this->entityManager->getConnection()->prepare(
            "SELECT * FROM $tabela WHERE (NRORG = :P_NRORG AND NRRATEIO = :P_NRRATEIO)"
            );
            
            $sql->execute(
                array(
                    'P_NRORG' => $nrorg,
                    'P_NRRATEIO' => $nrrateio
                )
            );
          
            return $sql->fetch();
    }


    public function selectPorcentagemEstrutura($nrorg, $nrrateio)
    {   
        $sql = $this->entityManager->getConnection()->prepare(
            "SELECT RAT.NRRATEIOESTRUT, ( RAT.NRPESO / SUM(RAT.NRPESO) OVER(PARTITION BY RAT.NRORG, RAT.NRRATEIO)) * 100 PERCENT FROM FPA_RATEIOESTRUT RAT WHERE RAT.NRORG = :P_NRORG AND RAT.NRRATEIO = :P_NRRATEIO"
            );
            
            $sql->execute(
                array(
                    'P_NRORG' => $nrorg,
                    'P_NRRATEIO' => $nrrateio
                )
            );
            
            return $sql->fetchAll();
    }

    public function selectPorcentagemOcupacao($nrorg, $nrrateio)
    {
        $sql = $this->entityManager->getConnection()->prepare(
            "SELECT RAT.NRRATEIOOCUP, ( RAT.NRPESO / SUM(RAT.NRPESO) OVER(PARTITION BY RAT.NRORG, RAT.NRRATEIO, RAT.NRESTRUTURAM)) * 100 PERCENT FROM FPA_RATEIOOCUP RAT WHERE RAT.NRORG = :P_NRORG AND RAT.NRRATEIO = :P_NRRATEIO"
            );
            
            $sql->execute(
                array(
                    'P_NRORG' => $nrorg,
                    'P_NRRATEIO' => $nrrateio
                )
            );
            
            return $sql->fetchAll();
    }

    public function setPorcentagemEstrutura($nrorg, $nrrateio, $nrrateioestrut, $vrporcentagem)
    {
        $sql = $this->entityManager->getConnection()->prepare("
            UPDATE FPA_RATEIOESTRUT 
            SET VRPORCENTAGEM     = :P_VRPORCENTAGEM 
            WHERE NRRATEIO     = :P_NRRATEIO
                AND NRRATEIOESTRUT  = :P_NRRATEIOESTRUT
                AND NRORG           = :P_NRORG");

            $sql->execute(
                array(
                    "P_VRPORCENTAGEM"  => $vrporcentagem,
                    "P_NRRATEIO"     => $nrrateio,
                    "P_NRRATEIOESTRUT"  => $nrrateioestrut,
                    "P_NRORG"           => $nrorg
                )
            );
    }

    public function setPorcentagemOcupacao($nrorg, $nrrateio, $nrrateioocup, $vrporcentagem)
    {
        $sql = $this->entityManager->getConnection()->prepare("
            UPDATE FPA_RATEIOOCUP 
            SET VRPORCENTAGEM     = :P_VRPORCENTAGEM 
            WHERE NRRATEIO     = :P_NRRATEIO
                AND NRRATEIOOCUP  = :P_NRRATEIOOCUP
                AND NRORG           = :P_NRORG");

            $sql->execute(
                array(
                    "P_VRPORCENTAGEM"  => $vrporcentagem,
                    "P_NRRATEIO"     => $nrrateio,
                    "P_NRRATEIOOCUP"  => $nrrateioocup,
                    "P_NRORG"           => $nrorg
                )
            );
    }

    public function getQntdVinculosPorEstrutGerenCargo($nrorg, $nrestrutlegal)
    {    
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        $sql = $this->entityManager->getConnection()->prepare(
           "SELECT VH.NRESTRUTGEREN, VH.NRCARGO, COUNT(VH.NRVINCULOM) QNTDVINCULOS
            FROM GPE_VINCULOM VM
            JOIN GPE_VINCULOH VH ON VH.NRORG      = VM.NRORG
                            AND VH.NRVINCULOM = VM.NRVINCULOM
                            AND VH.DTMESCOMPETENC = (SELECT MAX(VHH.DTMESCOMPETENC)
                                                       FROM GPE_VINCULOH VHH
                                                      WHERE VHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                        AND VHH.NRORG = VH.NRORG
                                                        AND VHH.NRVINCULOM = VH.NRVINCULOM)
            WHERE VM.NRORG = :P_NRORG
            AND VH.NRESTRUTLEGAL = :P_NRESTRUTLEGALRAT
             AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
            AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
              GROUP BY
                          VH.NRESTRUTGEREN,
                          VH.NRCARGO"
            );
            
            $sql->execute(
                array(
                    'P_NRORG' => $nrorg,
                    'P_NRESTRUTLEGALRAT' => $nrestrutlegal,
                    'P_DTMESCOMPETENC'   => $dtmescompetenc
                )
            );
            
            return $sql->fetchAll();
    }
}
