<?php

namespace HCM\PTO\Logic\AFD;

/**
 * Parametros da exportacao. Responsavel por armazenar as informacoes solicitadas para a exportacao
 * 
 * 
 */

class Parametros {
    /** @var string */
    private $nrOrg;
    /** @var string */
    private $nrOrgPadrao;
    /** @var string */
    private $dtMesCompetenc;
    /** @var string */
    private $nrPeriodoApuracao;
    /** @var string */
    private $dataInicio;
    /** @var string */
    private $dataFim;
    /** @var string */
    private $nrEstruturamEmpregador;
    /** @var string[] */
    private $listaVinculos;
    /** @var string[] */
    private $listaTiposVinculo;
    /** @var string[] */
    private $listaEstruturasLegais;
    /** @var string[] */
    private $listaEstruturasGerenciais;
        
    public function __construct(
        string  $nrOrg,
        string  $nrOrgPadrao,
        string  $dtMesCompetenc,
        string  $nrPeriodoApuracao,
        ?string $dataInicio,
        ?string $dataFim,
        string  $nrEstruturamEmpregador,
        array   $listaVinculos,
        array   $listaTiposVinculo,
        array   $listaEstruturasLegais,
        array   $listaEstruturasGerenciais
    ) {
        $this->nrOrg = $nrOrg;
        $this->nrOrgPadrao = $nrOrgPadrao;
        $this->dtMesCompetenc = $dtMesCompetenc;
        $this->nrPeriodoApuracao = $nrPeriodoApuracao;
        $this->dataInicio = substr($dataInicio, 0 , 10);
        $this->dataFim = substr($dataFim, 0 , 10);
        $this->nrEstruturamEmpregador = $nrEstruturamEmpregador;
        $this->listaVinculos = $listaVinculos;
        $this->listaTiposVinculo = $listaTiposVinculo;
        $this->listaEstruturasLegais = $listaEstruturasLegais;
        $this->listaEstruturasGerenciais = $listaEstruturasGerenciais;
    }
    
    /**
     * Retorna o numero da organizacao do parametro atual
     * @return string
     */
    public function getNrOrg(): string {
        return $this->nrOrg;
    }
    
    /**
     * Retorna o numero da organizacao padrao do parametro atual
     * @return string
     */
    public function getNrOrgPadrao(): string {
        return $this->nrOrgPadrao;
    }
    
    /**
     * Retorna a data de competencia do parametro atual
     * @return string
     */
    public function getDtMesCompetenc(): string {
        return $this->dtMesCompetenc;
    }
    
    /**
     * Retorna o periodo de apuracao do parametro atual
     * @return string
     */
    public function getNrPeriodoApuracao(): string {
        return $this->nrPeriodoApuracao;
    }
    
    /**
     * Retorna a data de inicio do parametro atual
     * @return string
     */
    public function getDataInicio(): ?string {
        return $this->dataInicio;
    }
    
    /**
     * Retorna a data fim do parametro atual
     * @return string
     */
    public function getDataFim(): ?string {
        return $this->dataFim;
    }
    
    /**
     * Retorna a estrutura do empregador do parametro atual
     * @return string
     */
    public function getNrEstruturamEmpregador(): string {
        return $this->nrEstruturamEmpregador;
    }
    
    /**
     * Retorna a lista de vinculos do parametro atual
     * @return array
     */
    public function getListaVinculos(): array {
        return $this->listaVinculos;
    }
    
    /**
     * Retorna a lista de vinculos concatenados separados por virgula do parametro atual
     * @return string
     */
    public function getVinculosString(): string {
        return implode(',', $this->listaVinculos);
    }
    
    /**
     * Retorna a lista de tipos de vinculo do parametro atual
     * @return array
     */
    public function getListaTiposVinculo(): array {
        return $this->listaTiposVinculo;
    }
    
    /**
     * Retorna a lista de tipos de vinculo concatenados separados por virgula do parametro atual
     * @return string
     */
    public function getTiposVinculoString(): string {
        return implode(',', $this->listaTiposVinculo);
    }
    
    /**
     * Retorna a lista de estruturas legais do parametro atual
     * @return array
     */
    public function getListaEstruturasLegais(): array {
        return $this->listaEstruturasLegais;
    }
    
    /**
     * Retorna a lista de estruturas legais concatenados separados por virgula do parametro atual
     * @return string
     */
    public function getEstruturasLegaisString(): string {
        return implode(',', $this->listaEstruturasLegais);
    }
    
    /**
     * Retorna a lista de estruturas gerenciais do parametro atual
     * @return array
     */
    public function getListaEstruturasGerenciais(): array {
        return $this->listaEstruturasGerenciais;
    }
    
    /**
     * Retorna a lista de estruturas gerenciais concatenados separados por virgula do parametro atual
     * @return string
     */
    public function getEstruturasGerenciaisString(): string {
        return implode(',', $this->listaEstruturasGerenciais);
    }
}