<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 21/07/2016
 * Time: 16:25
 */

namespace HCM\VAB\Controllers;

use Folha\Entities\FpaTipomoviment;
use Geral\Util\Util;
use HCM\Util\Messages;
use HCM\VAB\Services\LancamentoValeTransporteService;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class LancamentoValeTransporteController extends Crud {

    protected $dataSourceName = 'vab_histvalevinc';
    protected $dataSourceName_get = 'getHistvalevinc';

    protected $parameterBagColumns = array('P_NRORG', 'P_DTMESCOMPAPVALE');

    const GRAVAR_TOTAL = 'gravar_total';
    const GRAVAR_PARCIAL = 'gravar_parcial';
    const RETROCEDER_PARCIAL = 'retroceder_parcial';
    const RETROCEDER_TOTAL = 'retroceder_total';
    const CADASTRO_DIAS = 'cadastro_dias';
    const CADASTRO_AUTOMATICO = 'cadastro_automatico';
    const EXPORTAR = 'exportar';

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, LancamentoValeTransporteService $lancamentoValeTransporteService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->lancamentoValeTransporteService = $lancamentoValeTransporteService;
        parent::__construct($dataSourceManager);
    }

    public function getHistvalevinc (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_get);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function setHistvalevinc (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('DTMESCOMPAPVALE', $row)) {
                    if ($row['DTMESCOMPAPVALE'] === '') {
                        $row['DTMESCOMPAPVALE'] = $this->hcmParameters->getDtCompetencia();
                    }
                } else {
                    $row['DTMESCOMPAPVALE'] = $this->hcmParameters->getDtCompetencia();
                }

                $this->lancamentoValeTransporteService->verificaCalculo($row['NRVINCULOM'], $row['DTMESCOMPAPVALE'], $row['NRORG']);

                if (array_key_exists('NRCALCULOFOLHA', $row)) {
                    if ($row['NRCALCULOFOLHA'] === '') {
                        $row['NRCALCULOFOLHA'] = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_VALE_TRANSPORTE);
                    }
                } else {
                    $row['NRCALCULOFOLHA'] = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_VALE_TRANSPORTE);
                }

                if (array_key_exists('NRHISTVALEVINC', $row)) {
                    if ($row['NRHISTVALEVINC'] === '') {
                        $row['NRHISTVALEVINC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'VAB_HISTVALEVINC', 1, 12, true);
                    }
                } else {
                    $row['NRHISTVALEVINC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'VAB_HISTVALEVINC', 1, 12, true);
                }
                
                //Flag para marcar q foi ajustado manualmente.
                $row['IDAJUSTEMANUAL'] = 'S';
            }

            parent::save($request, $response);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function delete(DTO\Request\DataSet $request, DTO\Response $response){
        parent::delete($request, $response);
    }

    public function processaLancamentoValeTransporte(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                $action = $row['action'];
                $nrestrutsind = isset($row['NRESTRUTSIND']) ? $row['NRESTRUTSIND'] : null;
                $nrestrutlegal = isset($row['NRESTRUTLEGAL']) ? $row['NRESTRUTLEGAL'] : null;
                $nrestrutgeren = isset($row['NRESTRUTGEREN']) ? $row['NRESTRUTGEREN'] : null;
                $nrvinculom_list = $row['NRVINCULOM_LIST'];
                $nrtipolinhavinc = isset($row['NRTIPOLINHAVINC']) ? $row['NRTIPOLINHAVINC'] : null;
                $nrlinhatransp = isset($row['NRLINHATRANSP']) ? $row['NRLINHATRANSP'] : null;
                $nrsentidolinha = isset($row['NRSENTIDOLINHA']) ? $row['NRSENTIDOLINHA'] : null;
                $nrtarifa = isset($row['NRTARIFA']) ? $row['NRTARIFA'] : null;
                $qtapurvale = isset($row['QTAPURVALE']) ? $row['QTAPURVALE'] : null;
                $qtdiasvale = isset($row['QTDIASVALE']) ? $row['QTDIASVALE'] : null;
                $dtpagtovale = isset($row['DTPGTOVALE']) ? $row['DTPGTOVALE'] : null;
                $dtiniapurvale = isset($row['DTINIAPURVALE']) ? $row['DTINIAPURVALE'] : null;
                $dtfinapurvale = isset($row['DTFINAPURVALE']) ? $row['DTFINAPURVALE'] : null;
                $qtdiasacre = isset($row['QTDIASACRE']) ? $row['QTDIASACRE'] : 0;
                $qtdiasdesc = isset($row['QTDIASDESC']) ? $row['QTDIASDESC'] : 0;
                $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
                $nrorg = $this->hcmParameters->getNrorgTrabalho();

                $lista_nrvinculom = is_array($nrvinculom_list) ? "_".implode('_',array_values($nrvinculom_list))."_" : $nrvinculom_list;

                switch($action) {
                    case self::GRAVAR_TOTAL:
                        $this->lancamentoValeTransporteService->gravarTotal($dtmescompetenc, $nrorg, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $nrtipolinhavinc, $dtpagtovale);
                        break;
                    case self::GRAVAR_PARCIAL:
                        $this->lancamentoValeTransporteService->gravarParcial($dtmescompetenc, $nrorg, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $nrtipolinhavinc, $dtpagtovale);
                        break;
                    case self::RETROCEDER_PARCIAL:
                        $this->lancamentoValeTransporteService->retrocedeParcial($dtmescompetenc, $nrorg, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $nrtipolinhavinc);
                        break;
                    case self::RETROCEDER_TOTAL:
                        $this->lancamentoValeTransporteService->retrocedeTotal($dtmescompetenc, $nrorg, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $nrtipolinhavinc);
                        break;
                    case self::EXPORTAR:
                        $this->lancamentoValeTransporteService->exportarLote($dtmescompetenc, $nrorg, $nrestrutsind, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom);
                        break;
                    case self::CADASTRO_DIAS:
                        $this->lancamentoValeTransporteService->cadastroDias($lista_nrvinculom, $nrtipolinhavinc, $qtdiasacre, $qtdiasdesc);
                        break;
                    case self::CADASTRO_AUTOMATICO:
                        $this->lancamentoValeTransporteService->cadastroAutomatico($nrvinculom_list, $nrlinhatransp, $nrtarifa, $nrtipolinhavinc, $nrsentidolinha, $qtdiasvale, $dtpagtovale, $dtiniapurvale, $dtfinapurvale);
                        break;
                }
            }
        } catch (\Exception $e) {
            if ($e->getCode() === Messages::SUCCESS_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_MESSAGE));
            } elseif ($e->getCode() === Messages::ALERT_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_WARNING));
            } elseif ($e->getCode() === Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}